# pylint: disable=dangerous-default-value
# pylint: disable=line-too-long

import base64
import uuid

from joinnector.wrapper.request_wrapper import request_wrapper

from joinnector.helper.collection_helper import CollectioHelper
from joinnector.helper.constant_helper import ConstantHelper


class BaseSDKService(object):
    def __init__(self, name):
        self.name = name

    def get_infos(self, endpoint):
        apimapopts = ConstantHelper.get_setting_constant().API_MAP.get(self.name)

        base_url = ConstantHelper.get_setting_constant().API_PROD_BASE_URL if request_wrapper.get_wrapper(
        ).mode == "prod" else ConstantHelper.get_setting_constant().API_DEV_BASE_URL

        url = CollectioHelper.process_key_join(value=[base_url, "/api/open", endpoint], separator="")
        headers = { **ConstantHelper.get_setting_constant().API_BASE_HEADER}
        params = {}

        return request_wrapper.get_wrapper().process_axios_get(
            url=url, headers=headers, params=params)

    def create(self, payload, action="create"):
        apimapopts = ConstantHelper.get_setting_constant().API_MAP.get(self.name)

        base_url = ConstantHelper.get_setting_constant().API_PROD_BASE_URL if request_wrapper.get_wrapper(
        ).mode == "prod" else ConstantHelper.get_setting_constant().API_DEV_BASE_URL

        url = CollectioHelper.process_key_join(value=[base_url, apimapopts.get(
            action).get("prefix"), apimapopts.get(action).get("endpoint")], separator="")
        headers = { **ConstantHelper.get_setting_constant().API_BASE_HEADER}
        params = {}
        attributes = payload

        token = "%(name)s:%(pass)s" % {
            "name": request_wrapper.get_wrapper().key, "pass": request_wrapper.get_wrapper().secret}
        headers.update({"authorization": "Basic " +
                        base64.b64encode(token.encode("utf-8")).decode('ascii')})

        headers.update(
            {"x-apikey": request_wrapper.get_wrapper().key})

        return request_wrapper.get_wrapper().process_axios_post(
            url=url, headers=headers, params=params, data=attributes)

    def get(self, by_id, action="get"):
        apimapopts = ConstantHelper.get_setting_constant().API_MAP.get(self.name)

        base_url = ConstantHelper.get_setting_constant().API_PROD_BASE_URL if request_wrapper.get_wrapper(
        ).mode == "prod" else ConstantHelper.get_setting_constant().API_DEV_BASE_URL

        url = CollectioHelper.process_key_join(value=[base_url, apimapopts.get(action).get(
            "prefix"), apimapopts.get(action).get("endpoint")], separator="").replace("{id}", by_id)
        headers = { **ConstantHelper.get_setting_constant().API_BASE_HEADER}

        token = "%(name)s:%(pass)s" % {
            "name": request_wrapper.get_wrapper().key, "pass": request_wrapper.get_wrapper().secret}
        headers.update({"authorization": "Basic " +
                        base64.b64encode(token.encode("utf-8")).decode('ascii')})

        headers.update(
            {"x-apikey": request_wrapper.get_wrapper().key})

        return request_wrapper.get_wrapper().process_axios_get(
            url=url, headers=headers, params={})

    def get_by(self, by_key, by_value, swap_id=None, action="get"):
        apimapopts = ConstantHelper.get_setting_constant().API_MAP.get(self.name)

        base_url = ConstantHelper.get_setting_constant().API_PROD_BASE_URL if request_wrapper.get_wrapper(
        ).mode == "prod" else ConstantHelper.get_setting_constant().API_DEV_BASE_URL

        url = CollectioHelper.process_key_join(value=[base_url, apimapopts.get(action).get(
            "prefix"), apimapopts.get(action).get("endpoint")], separator="").replace("{id}", str(uuid.uuid4()))
        headers = { **ConstantHelper.get_setting_constant().API_BASE_HEADER}
        params = {}

        params.update({by_key: by_value})
        if swap_id is not None:
            params.update({"swap_id": swap_id})

        token = "%(name)s:%(pass)s" % {
            "name": request_wrapper.get_wrapper().key, "pass": request_wrapper.get_wrapper().secret}
        headers.update({"authorization": "Basic " +
                        base64.b64encode(token.encode("utf-8")).decode('ascii')})

        headers.update(
            {"x-apikey": request_wrapper.get_wrapper().key})

        headers.update(
            {"content-type": "application/x-www-form-urlencoded"})

        return request_wrapper.get_wrapper().process_axios_get(
            url=url, headers=headers, params=params)

    def save(self, by_id, payload, action="save"):
        apimapopts = ConstantHelper.get_setting_constant().API_MAP.get(self.name)

        base_url = ConstantHelper.get_setting_constant().API_PROD_BASE_URL if request_wrapper.get_wrapper(
        ).mode == "prod" else ConstantHelper.get_setting_constant().API_DEV_BASE_URL

        url = CollectioHelper.process_key_join(value=[base_url, apimapopts.get(action).get(
            "prefix"), apimapopts.get(action).get("endpoint")], separator="").replace("{id}", by_id)
        headers = { **ConstantHelper.get_setting_constant().API_BASE_HEADER}
        attributes = payload

        token = "%(name)s:%(pass)s" % {
            "name": request_wrapper.get_wrapper().key, "pass": request_wrapper.get_wrapper().secret}
        headers.update({"authorization": "Basic " +
                        base64.b64encode(token.encode("utf-8")).decode('ascii')})

        headers.update(
            {"x-apikey": request_wrapper.get_wrapper().key})

        return request_wrapper.get_wrapper().process_axios_put(
            url=url, headers=headers, params={}, data=attributes)

    def delete(self, by_id, action="get"):
        apimapopts = ConstantHelper.get_setting_constant().API_MAP.get(self.name)

        base_url = ConstantHelper.get_setting_constant().API_PROD_BASE_URL if request_wrapper.get_wrapper(
        ).mode == "prod" else ConstantHelper.get_setting_constant().API_DEV_BASE_URL

        url = CollectioHelper.process_key_join(value=[base_url, apimapopts.get(action).get(
            "prefix"), apimapopts.get(action).get("endpoint")], separator="").replace("{id}", by_id)
        headers = { **ConstantHelper.get_setting_constant().API_BASE_HEADER}

        token = "%(name)s:%(pass)s" % {
            "name": request_wrapper.get_wrapper().key, "pass": request_wrapper.get_wrapper().secret}
        headers.update({"authorization": "Basic " +
                        base64.b64encode(token.encode("utf-8")).decode('ascii')})

        headers.update(
            {"x-apikey": request_wrapper.get_wrapper().key})

        return request_wrapper.get_wrapper().process_axios_delete(
            url=url, headers=headers, params={})

    def fetch(self, by_filter={}, action="fetch"):

        apimapopts = ConstantHelper.get_setting_constant().API_MAP.get(self.name)

        base_url = ConstantHelper.get_setting_constant().API_PROD_BASE_URL if request_wrapper.get_wrapper(
        ).mode == "prod" else ConstantHelper.get_setting_constant().API_DEV_BASE_URL

        url = CollectioHelper.process_key_join(value=[base_url, apimapopts.get(
            action).get("prefix"), apimapopts.get(action).get("endpoint")], separator="")
        headers = { **ConstantHelper.get_setting_constant().API_BASE_HEADER}
        params = { "page": 1, "limit": 20, **by_filter }

        token = "%(name)s:%(pass)s" % {
            "name": request_wrapper.get_wrapper().key, "pass": request_wrapper.get_wrapper().secret}
        headers.update({"authorization": "Basic " +
                        base64.b64encode(token.encode("utf-8")).decode('ascii')})

        headers.update(
            {"x-apikey": request_wrapper.get_wrapper().key})

        return request_wrapper.get_wrapper().process_axios_get(
            url=url, headers=headers, params=params)



# pylint: disable=useless-super-delegation

from joinnector.service.base_sdk_service import BaseSDKService


class LeadService(BaseSDKService):
    def __init__(self, name):
        super().__init__(name)

    def get_by_customer_id(self, customer_id, swap_id=None):
        return super().get_by("customer_id", customer_id, swap_id)

    def get_by_email(self, email, swap_id=None):
        return super().get_by("email", email, swap_id)

    def get_by_mobile(self, mobile, swap_id=None):
        return super().get_by("mobile", mobile, swap_id)


lead_service = LeadService("lead")



# pylint: disable=attribute-defined-outside-init

from joinnector.client.logging_client import LoggingClient


class LoggingWrapper(object):
    def init(self):
        self.process_common_wrapper()

    def process_common_wrapper(self):
        self.logging_client = LoggingClient()

    def get_wrapper(self):
        return self.logging_client


logging_wrapper = LoggingWrapper()



# client
from joinnector.client.delegate_client import DelegateClient

from joinnector.wrapper.security_wrapper import sucurity_wrapper
from joinnector.wrapper.logging_wrapper import logging_wrapper
from joinnector.wrapper.request_wrapper import request_wrapper

# service
from joinnector.service.coupon_service import coupon_service
from joinnector.service.currency_service import currency_service
from joinnector.service.deal_service import deal_service
from joinnector.service.offer_service import offer_service
from joinnector.service.store_service import store_service
from joinnector.service.lead_service import lead_service
from joinnector.service.notification_service import notification_service
from joinnector.service.review_service import review_service
from joinnector.service.setting_service import setting_service
from joinnector.service.swap_service import swap_service
from joinnector.service.task_service import task_service
from joinnector.service.taskactivity_service import taskactivity_service
from joinnector.service.surprise_service import surprise_service
from joinnector.service.surpriseactivity_service import surpriseactivity_service
from joinnector.service.wallet_service import wallet_service
from joinnector.service.wallettransaction_service import wallettransaction_service


class SDK(object):
    def __init__(self, key, secret, mode="prod"):
        self.init_wrappers(key=key, secret=secret, mode=mode)

    def init_wrappers(self, key, secret, mode):
        sucurity_wrapper.init()
        logging_wrapper.init()
        request_wrapper.init(key=key, secret=secret, mode=mode)

    @staticmethod
    def get_delegate_client():
        return DelegateClient

    def get_coupon_service(self):
        return coupon_service

    def get_currency_service(self):
        return currency_service

    def get_deal_service(self):
        return deal_service

    def get_offer_service(self):
        return offer_service

    def get_store_service(self):
        return store_service

    def get_lead_service(self):
        return lead_service

    def get_notification_service(self):
        return notification_service

    def get_review_service(self):
        return review_service

    def get_setting_service(self):
        return setting_service

    def get_swap_service(self):
        return swap_service

    def get_task_service(self):
        return task_service

    def get_taskactivity_service(self):
        return taskactivity_service

    def get_surprise_service(self):
        return surprise_service

    def get_surpriseactivity_service(self):
        return surpriseactivity_service

    def get_wallet_service(self):
        return wallet_service

    def get_wallettransaction_service(self):
        return wallettransaction_service



SERVICE_NAME = "nectorsdk"

API_PROD_BASE_URL = "https://platform.nector.io"
API_DEV_BASE_URL = "https://devplatform.nector.io"

API_BASE_HEADER = {
    "accept": "application/json",
    "content-type": "application/json",
    "x-source": "web"
}

API_MAP = {
    "coupon": {
        "create": {"endpoint": "/coupons", "prefix": "/api/v2/merchant",  "has_signature": True},
        "get": {"endpoint": "/coupons/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/coupons", "prefix": "/api/v2/merchant", }
    },
    "currency": {
        "get": {"endpoint": "/currencies/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/currencies", "prefix": "/api/v2/merchant", }
    },
    "deal": {
        "reward": {"endpoint": "/dealrewards", "prefix": "/api/v2/merchant",  "has_signature": True},
        "get": {"endpoint": "/deals/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/deals", "prefix": "/api/v2/merchant", }
    },
    "offer": {
        "redeem": {"endpoint": "/offerredeems", "prefix": "/api/v2/merchant",  "has_signature": True},
        "get": {"endpoint": "/offers/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/offers", "prefix": "/api/v2/merchant", }
    },
    "store": {
        "get": {"endpoint": "/stores/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/stores", "prefix": "/api/v2/merchant", }
    },
    "lead": {
        "create": {"endpoint": "/leads", "prefix": "/api/v2/merchant",  "has_signature": True},
        "get": {"endpoint": "/leads/{id}", "prefix": "/api/v2/merchant", },
        "save": {"endpoint": "/leads/{id}", "prefix": "/api/v2/merchant",  "has_signature": True},
    },
    "notification": {
        "get": {"endpoint": "/notifications/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/notifications", "prefix": "/api/v2/merchant", }
    },
    "review": {
        "create": {"endpoint": "/reviews", "prefix": "/api/v2/merchant",  "has_signature": True},
        "get": {"endpoint": "/reviews/{id}", "prefix": "/api/v2/merchant", },
        "save": {"endpoint": "/reviews/{id}", "prefix": "/api/v2/merchant",  "has_signature": True},
        "delete": {"endpoint": "/reviews/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/reviews", "prefix": "/api/v2/merchant", }
    },
    "setting": {
        "get": {"endpoint": "/settings/{id}", "prefix": "/api/v2/merchant", }
    },
    "swap": {
        "create": {"endpoint": "/swaps", "prefix": "/api/v2/merchant",  "has_signature": True},
        "get": {"endpoint": "/swaps/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/swaps", "prefix": "/api/v2/merchant", }
    },
    "task": {
        "get": {"endpoint": "/tasks/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/tasks", "prefix": "/api/v2/merchant", }
    },
    "taskactivity": {
        "create": {"endpoint": "/taskactivities", "prefix": "/api/v2/merchant",  "has_signature": True},
        "get": {"endpoint": "/taskactivities/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/taskactivities", "prefix": "/api/v2/merchant", }
    },
    "surprise": {
        "get": {"endpoint": "/surprises/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/surprises", "prefix": "/api/v2/merchant", }
    },
    "surpriseactivity": {
        "create": {"endpoint": "/surpriseactivities", "prefix": "/api/v2/merchant",  "has_signature": True},
        "get": {"endpoint": "/surpriseactivities/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/surpriseactivities", "prefix": "/api/v2/merchant", }
    },
    "wallet": {
        "create": {"endpoint": "/wallets", "prefix": "/api/v2/merchant",  "has_signature": True},
        "get": {"endpoint": "/wallets/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/wallets", "prefix": "/api/v2/merchant", }
    },
    "wallettransaction": {
        "create": {"endpoint": "/wallettransactions", "prefix": "/api/v2/merchant",  "has_signature": True},
        "get": {"endpoint": "/wallettransactions/{id}", "prefix": "/api/v2/merchant", },
        "fetch": {"endpoint": "/wallettransactions", "prefix": "/api/v2/merchant", }
    }
}



# pylint: disable=useless-super-delegation

from joinnector.service.base_sdk_service import BaseSDKService


class OfferService(BaseSDKService):
    def __init__(self, name):
        super().__init__(name)

    def redeem(self, payload):
        return super().create(payload, "redeem")

    def get_by_sku(self, sku):
        return super().get_by("sku", sku)


offer_service = OfferService("offer")



# pylint: disable=attribute-defined-outside-init

from joinnector.client.security_client import SecurityClient


class SecurityWrapper(object):
    def init(self):
        self.process_common_wrapper()

    def process_common_wrapper(self):
        self.security_client = SecurityClient()

    def get_wrapper(self):
        return self.security_client


sucurity_wrapper = SecurityWrapper()



# pylint: disable=useless-super-delegation

from joinnector.service.base_sdk_service import BaseSDKService


class TaskActivityService(BaseSDKService):
    def __init__(self, name):
        super().__init__(name)


taskactivity_service = TaskActivityService("taskactivity")



import logging

from joinnector.helper.constant_helper import ConstantHelper


class LoggingClient(object):
    def __init__(self):
        logging.basicConfig(
            format="%(asctime)s - %(name)s - %(levelname)s - %(message)s", level=10)

    def process_ie_log(self, **kwargs):
        kwargs.update(
            {"service": ConstantHelper.get_setting_constant().SERVICE_NAME})
        getattr(logging, kwargs.get("level", "debug"))(
            **kwargs, stack_info=True)



# pylint: disable=attribute-defined-outside-init

from joinnector.client.request_client import RequestClient


class RequestWrapper(object):
    def init(self, key, secret, mode):
        self.process_common_wrapper(key, secret, mode)

    def process_common_wrapper(self, key, secret, mode):
        self.request_client = RequestClient(
            key=key, secret=secret, mode=mode)

    def get_wrapper(self):
        return self.request_client


request_wrapper = RequestWrapper()



# pylint: disable=useless-super-delegation

from joinnector.service.base_sdk_service import BaseSDKService


class WalletTransactionService(BaseSDKService):
    def __init__(self, name):
        super().__init__(name)


wallettransaction_service = WalletTransactionService("wallettransaction")



# pylint: disable=no-self-use
# pylint: disable=dangerous-default-value

import json
import urllib.parse
import requests

from joinnector.helper.constant_helper import ConstantHelper


class RequestClient(object):
    '''
    key - api key
    secret - api secret
    '''

    def __init__(self, key, secret, mode):
        self.key = key
        self.secret = secret
        self.mode = mode

    def change_mode(self):
        self.mode="prod"

    def process_axios_get(self, url, headers, params={}):
        headers = {**headers,  **
                   ConstantHelper.get_setting_constant().API_BASE_HEADER}

        if headers["content-type"] == "application/x-www-form-urlencoded":
            headers.update(
                {"content-type": "application/x-www-form-urlencoded"})

        params = urllib.parse.urlencode(params, doseq=True)

        return requests.get(url, headers=headers, params=params)

    def process_axios_put(self, url, headers, params={}, data={}):
        headers = {**headers,  **
                   ConstantHelper.get_setting_constant().API_BASE_HEADER}

        if headers["content-type"] == "application/x-www-form-urlencoded":
            headers.update(
                {"content-type": "application/x-www-form-urlencoded"})

        params = urllib.parse.urlencode(params, doseq=True)

        return requests.put(url, headers=headers, params=params, data=json.dumps(data))

    def process_axios_delete(self, url, headers, params={}):
        headers = {**headers,  **
                   ConstantHelper.get_setting_constant().API_BASE_HEADER}

        if headers["content-type"] == "application/x-www-form-urlencoded":
            headers.update(
                {"content-type": "application/x-www-form-urlencoded"})

        params = urllib.parse.urlencode(params, doseq=True)

        return requests.delete(url, headers=headers, params=params)

    def process_axios_post(self, url, headers, params={}, data={}):
        headers = {**headers,  **
                   ConstantHelper.get_setting_constant().API_BASE_HEADER}

        if headers["content-type"] == "application/x-www-form-urlencoded":
            headers.update(
                {"content-type": "application/x-www-form-urlencoded"})

        params = urllib.parse.urlencode(params, doseq=True)

        print(headers)

        return requests.post(url, headers=headers, params=params, data=json.dumps(data))



# pylint: disable=dangerous-default-value
import json


class CollectioHelper(object):
    @staticmethod
    def process_key_join(value=[], separator="_"):
        return separator.join(value)

    @staticmethod
    def process_serialize_data(value):
        if isinstance(value, dict) is True:
            return json.dumps(value, check_circular=True)

        return ""

    @staticmethod
    def process_deserialize_data(value):
        if isinstance(value, str) is True:
            return json.loads(value)

        return dict()



import hmac
import hashlib


class SecurityClient(object):
    def process_hmac_signature(self, value, password):
        return hmac.new(password.encode('utf8'), value.encode('utf8'), hashlib.sha256).hexdigest()



from setuptools import setup, find_packages

setup(
    name='joinnector',         # How you named your package folder (MyLib)
    packages=find_packages(),
    version='6.2',      # Start with a small number and increase it with every change you make
    # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    license='MIT',
    # Give a short description about your library
    description='Nector python SDK (nector provides a suite of plug-n-play APIs & SDK focused on rewarding customers, driving customer engagements and improving sales, for the businesses.)',
    author='Tech, Nector',                   # Type in your name
    author_email='tech@nector.io',      # Type in your E-Mail
    url='https://github.com/joinnector/joinnector-python-sdk',
    # I explain this later on
    download_url='https://github.com/joinnector/joinnector-python-sdk/archive/6.2.tar.gz',
    # Keywords that define your package best
    keywords=['nector', 'reward', 'loyalty', 'coins', 'wallets', 'deals'],
    install_requires=[            # I get to this in a second
        'requests',
    ],
    classifiers=[
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Development Status :: 4 - Beta',
        # Define that your audience are developers
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



# pylint: disable=useless-super-delegation

from joinnector.service.base_sdk_service import BaseSDKService


class CurrencyService(BaseSDKService):
    def __init__(self, name):
        super().__init__(name)

    def get_by_currency_code(self, currency_code):
        return super().get_by("currency_code", currency_code)


currency_service = CurrencyService("currency")



class DelegateClient(object):
    def __init__(self, nector_sdk_instance, not_allowed_controller_method_names=None):
        self.nector_sdk_instance = nector_sdk_instance

        '''
        For security purpose these methods can not be triggered from client calls
        // To whitelist calls directly from client side, remove the method name from the array
        // It is requested to call the "not_allowed_controller_method_names" only from other backend functions (idealy they should be called while performing business operations) since they cause quota consumption on nector.
        '''

        self.not_allowed_controller_method_names = ["reward_deals", "redeem_offers" "create_leads", "save_leads", "get_subscriptions", "create_taskactivities",
                                                    "create_wallets", "create_wallettransactions"] if (not_allowed_controller_method_names is None) else not_allowed_controller_method_names

    '''
    Delegate to correct method, designed to be called directly from route or controoler handler function
    Client request body structure must follow the below pattern
        req.body = {
            method: string<"name_of_method_where_to_delegate_the_call">,
            params: object<{params_of_that_method}>,
            query: object<{query_of_that_method}>
            body: object<{body_of_that_method}>
        }
    '''

    def delegate_method(self, json_data):
        if json_data is None:
            raise Exception("Something went wrong, body is required")

        method = json_data.get("method", None)
        params = json_data.get("params", {})
        query = json_data.get("query", {})
        body = json_data.get("body", {})

        if method is None or method == "delegate_method" or method in self.not_allowed_controller_method_names or getattr(self, method, None) is None:
            raise Exception("Something went wrong, please try after sometime")

        return getattr(self, method)(params, query, body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1systeminfos/get for request signature and other information
    def get_systeminfos(self, params, query, body):
        return self.nector_sdk_instance.get_coupon_service().get_infos("/systeminfos")



    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1coupons/post for request signature and other information
    def create_coupons(self, params, query, body):
        return self.nector_sdk_instance.get_coupon_service().create(body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1coupons~1%7Bid%7D/get for request signature and other information
    def get_coupons(self, params, query, body):
        return self.nector_sdk_instance.get_coupon_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1coupons/get for request signature and other information
    def fetch_coupons(self, params, query, body):
        return self.nector_sdk_instance.get_coupon_service().fetch(query or {})



    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1currencies~1%7Bid%7D/get for request signature and other information
    def get_currencies(self, params, query, body):
        return self.nector_sdk_instance.get_currency_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1currencies~1%7Bid%7D/get for request signature and other information
    def get_currencies_by_currency_code(self, params, query, body):
        return self.nector_sdk_instance.get_currency_service().get_by_currency_code(query.get("currency_code"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1currencies/get for request signature and other information
    def fetch_currencies(self, params, query, body):
        return self.nector_sdk_instance.get_currency_service().fetch(query or {})



    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1dealrewards/post for request signature and other information
    def reward_deals(self, params, query, body):
        return self.nector_sdk_instance.get_deal_service().reward(body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1deals~1%7Bid%7D/get for request signature and other information
    def get_deals(self, params, query, body):
        return self.nector_sdk_instance.get_deal_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1deals~1%7Bid%7D/get for request signature and other information
    def get_deals_by_sku(self, params, query, body):
        return self.nector_sdk_instance.get_deal_service().get_by_sku(query.get("sku"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1deals/get for request signature and other information
    def fetch_deals(self, params, query, body):
        return self.nector_sdk_instance.get_deal_service().fetch(query or {})


    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1offerredeems/post for request signature and other information
    def redeem_offers(self, params, query, body):
        return self.nector_sdk_instance.get_offer_service().redeem(body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1offers~1%7Bid%7D/get for request signature and other information
    def get_offers(self, params, query, body):
        return self.nector_sdk_instance.get_offer_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1offers~1%7Bid%7D/get for request signature and other information
    def get_offers_by_sku(self, params, query, body):
        return self.nector_sdk_instance.get_offer_service().get_by_sku(query.get("sku"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1offers/get for request signature and other information
    def fetch_offers(self, params, query, body):
        return self.nector_sdk_instance.get_offer_service().fetch(query or {})



    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1stores~1%7Bid%7D/get for request signature and other information
    def get_stores(self, params, query, body):
        return self.nector_sdk_instance.get_store_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1stores~1%7Bid%7D/get for request signature and other information
    def get_stores_by_sku(self, params, query, body):
        return self.nector_sdk_instance.get_store_service().get_by_sku(query.get("sku"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1stores/get for request signature and other information
    def fetch_stores(self, params, query, body):
        return self.nector_sdk_instance.get_store_service().fetch(query or {})



    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1leads/post for request signature and other information
    def create_leads(self, params, query, body):
        return self.nector_sdk_instance.get_lead_service().create(body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1leads~1%7Bid%7D/get for request signature and other information
    def get_leads(self, params, query, body):
        return self.nector_sdk_instance.get_lead_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1leads~1%7Bid%7D/get for request signature and other information
    def get_leads_by_email(self, params, query, body):
        return self.nector_sdk_instance.get_lead_service().get_by_email(query.get("email"), query.get("swap_id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1leads~1%7Bid%7D/get for request signature and other information
    def get_leads_by_mobile(self, params, query, body):
        return self.nector_sdk_instance.get_lead_service().get_by_mobile(query.get("mobile"), query.get("swap_id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1leads~1%7Bid%7D/get for request signature and other information
    def get_leads_by_customer_id(self, params, query, body):
        return self.nector_sdk_instance.get_lead_service().get_by_customer_id(query.get("customer_id"), query.get("swap_id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1leads~1%7Bid%7D/put for request signature and other information
    def save_leads(self, params, query, body):
        return self.nector_sdk_instance.get_lead_service().save(params.get("id"), body)




    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1notifications~1%7Bid%7D/get for request signature and other information
    def get_notifications(self, params, query, body):
        return self.nector_sdk_instance.get_notification_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1notifications/get for request signature and other information
    def fetch_notifications(self, params, query, body):
        return self.nector_sdk_instance.get_notification_service().fetch(query or {})




    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1reviews/post for request signature and other information
    def create_reviews(self, params, query, body):
        return self.nector_sdk_instance.get_review_service().create(body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1reviews~1%7Bid%7D/get for request signature and other information
    def get_reviews(self, params, query, body):
        return self.nector_sdk_instance.get_review_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1reviews~1%7Bid%7D/delete for request signature and other information
    def delete_reviews(self, params, query, body):
        return self.nector_sdk_instance.get_review_service().delete(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1reviews~1%7Bid%7D/put for request signature and other information
    def save_reviews(self, params, query, body):
        return self.nector_sdk_instance.get_review_service().save(params.get("id"), body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1reviews/get for request signature and other information
    def fetch_reviews(self, params, query, body):
        return self.nector_sdk_instance.get_review_service().fetch(query or {})




    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1settings~1%7Bid%7D/get for request signature and other information
    def get_subscriptions(self, params, query, body):
        return self.nector_sdk_instance.get_subscription_service().get(params.get("id"))



    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1swaps/post for request signature and other information
    def create_swaps(self, params, query, body):
        return self.nector_sdk_instance.get_swap_service().create(body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1swaps~1%7Bid%7D/get for request signature and other information
    def get_swaps(self, params, query, body):
        return self.nector_sdk_instance.get_swap_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1swaps~1%7Bid%7D/get for request signature and other information
    def get_swaps_by_sku(self, params, query, body):
        return self.nector_sdk_instance.get_swap_service().get_by_sku(query.get("sku"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1swaps/get for request signature and other information
    def fetch_swaps(self, params, query, body):
        return self.nector_sdk_instance.get_swap_service().fetch(query or {})




    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1tasks~1%7Bid%7D/get for request signature and other information
    def get_tasks(self, params, query, body):
        return self.nector_sdk_instance.get_task_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1tasks~1%7Bid%7D/get for request signature and other information
    def get_tasks_by_sku(self, params, query, body):
        return self.nector_sdk_instance.get_task_service().get_by_sku(query.get("sku"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1tasks/get for request signature and other information
    def fetch_tasks(self, params, query, body):
        return self.nector_sdk_instance.get_task_service().fetch(query or {})




    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1taskactivities/post for request signature and other information
    def create_taskactivities(self, params, query, body):
        return self.nector_sdk_instance.get_taskactivity_service().create(body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1taskactivities~1%7Bid%7D/get for request signature and other information
    def get_taskactivities(self, params, query, body):
        return self.nector_sdk_instance.get_taskactivity_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1taskactivities/get for request signature and other information
    def fetch_taskactivities(self, params, query, body):
        return self.nector_sdk_instance.get_taskactivity_service().fetch(query or {})


    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1surprises~1%7Bid%7D/get for request signature and other information
    def get_surprises(self, params, query, body):
        return self.nector_sdk_instance.get_surprise_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1surprises~1%7Bid%7D/get for request signature and other information
    def get_surprises_by_sku(self, params, query, body):
        return self.nector_sdk_instance.get_surprise_service().get_by_sku(query.get("sku"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1surprises/get for request signature and other information
    def fetch_surprises(self, params, query, body):
        return self.nector_sdk_instance.get_surprise_service().fetch(query or {})




    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1surpriseactivities/post for request signature and other information
    def create_surpriseactivities(self, params, query, body):
        return self.nector_sdk_instance.get_surpriseactivity_service().create(body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1surpriseactivities~1%7Bid%7D/get for request signature and other information
    def get_surpriseactivities(self, params, query, body):
        return self.nector_sdk_instance.get_surpriseactivity_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1surpriseactivities/get for request signature and other information
    def fetch_surpriseactivities(self, params, query, body):
        return self.nector_sdk_instance.get_surpriseactivity_service().fetch(query or {})




    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1wallets/post for request signature and other information
    def create_wallets(self, params, query, body):
        return self.nector_sdk_instance.get_wallet_service().create(body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1wallets~1%7Bid%7D/get for request signature and other information
    def get_wallets(self, params, query, body):
        return self.nector_sdk_instance.get_wallet_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1wallets/get for request signature and other information
    def fetch_wallets(self, params, query, body):
        return self.nector_sdk_instance.get_wallet_service().fetch(query or {})




    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1wallettransactions/post for request signature and other information
    def create_wallettransactions(self, params, query, body):
        return self.nector_sdk_instance.get_wallettransaction_service().create(body)

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1wallettransactions~1%7Bid%7D/get for request signature and other information
    def get_wallettransactions(self, params, query, body):
        return self.nector_sdk_instance.get_wallettransaction_service().get(params.get("id"))

    # Please refer https://apidocs.nector.io/docs/api-docs/api_reference_oas3.yaml/paths/~1wallettransactions/get for request signature and other information
    def fetch_wallettransactions(self, params, query, body):
        return self.nector_sdk_instance.get_wallettransaction_service().fetch(query or {})
